Package: ggbreak
Title: Set Axis Break for 'ggplot2'
Version: 0.1.7
Authors@R: c(
       person("Guangchuang", "Yu",     email = "guangchuangyu@gmail.com", role = c("aut", "cre", "cph"), comment = c(ORCID = "0000-0002-6485-8781")),
       person("Shuangbin", "Xu",       email = "xshuangbin@163.com",      role = "aut", comment = c(ORCID="0000-0003-3513-5362"))
       )
Description: An implementation of scale functions for setting axis breaks of a 'gg' plot (S Xu (2021) <doi:10.3389/fgene.2021.774846>).
Imports: ggfun (>= 0.1.1), grid, ggplot2, ggplotify (>= 0.0.7), aplot
        (>= 0.1.5), rlang, stats, yulab.utils (>= 0.1.9)
Suggests: cowplot, ggimage, knitr, patchwork, pillar, prettydoc,
        rmarkdown, testthat
VignetteBuilder: knitr
License: Artistic-2.0
Encoding: UTF-8
URL: https://github.com/YuLab-SMU/ggbreak
BugReports: https://github.com/YuLab-SMU/ggbreak/issues
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-03-18 07:35:11 UTC; HUAWEI
Author: Guangchuang Yu [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Shuangbin Xu [aut] (ORCID: <https://orcid.org/0000-0003-3513-5362>)
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-18 08:02:04 UTC

# Generated by roxygen2: do not edit by hand

S3method(ggplot_add,gg)
S3method(ggplot_add,ggbreak)
S3method(ggplot_add,ggbreak_params)
S3method(ggplot_add,ggcut)
S3method(ggplot_add,ggcut_params)
S3method(ggplot_add,ggwrap)
S3method(ggplot_add,wrap_params)
S3method(grid.draw,ggbreak)
S3method(grid.draw,ggcut)
S3method(grid.draw,ggwrap)
S3method(plot,ggbreak)
S3method(print,ggbreak)
S3method(print,ggcut)
S3method(print,ggwrap)
export(scale_wrap)
export(scale_x_break)
export(scale_x_cut)
export(scale_y_break)
export(scale_y_cut)
importFrom(aplot,plot_list)
importFrom(ggfun,ggbreak2ggplot)
importFrom(ggfun,is.ggbreak)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,ggplot_add)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggplotify,as.ggplot)
importFrom(grid,grid.draw)
importFrom(rlang,abort)
importFrom(stats,setNames)
importFrom(utils,packageDescription)

#' @importFrom utils packageDescription
#' @importFrom yulab.utils yulab_msg
.onAttach <- function(libname, pkgname) {
    packageStartupMessage(yulab_msg(pkgname))

    citation <- paste0(paste0("If you use ", 
                              pkgname, 
                              " in published research, please cite the following paper:\n\n"
                              ),
                       ggbreak_citation(), collapse = "\n")

    packageStartupMessage(paste0(strwrap(pillar::style_subtle(citation)), collapse="\n"))
}


ggbreak_citation <- function() {
    paste0("S Xu, M Chen, T Feng, L Zhan, L Zhou, G Yu. ",
           "Use ggbreak to effectively utilize plotting space to deal with large datasets and outliers. ",
           "Frontiers in Genetics. 2021, 12:774846. doi: 10.3389/fgene.2021.774846\n"
           )
}


.ggbreak <- yulab.utils::get_cache()
.ggbreak$scale_break <- function(axis, breaks, scales, ticklabels=NULL, expand=TRUE, space = .1, symbol = NULL) {
    call_stack <- sys.calls()
    for (call in call_stack) {
        if (is.function(call[[1]]) && is.null(attr(call[[1]], "name"))) {
            stop("invalid called.")
        }
    }

    structure(list(axis = axis, breaks = breaks, scales=scales,
                   ticklabels=ticklabels, expand = expand, space = space, symbol = symbol),
              class = "ggbreak_params")
}

#' @importFrom grid grid.draw
#' @method grid.draw ggbreak
#' @importFrom ggplotify as.ggplot
#' @importFrom ggplot2 coord_cartesian
#' @importFrom ggplot2 xlab
#' @importFrom ggplot2 ylab
#' @importFrom ggplot2 theme
#' @importFrom ggplot2 element_blank
#' @importFrom ggplot2 element_text
#' @importFrom ggplot2 scale_x_continuous
#' @importFrom ggplot2 scale_y_continuous
#' @importFrom aplot plot_list
#' @importFrom stats setNames

## Dual-axis break rendering: creates an Nx * Ny 2D grid of subplots
render_dual_break <- function(x, axis_break_x, axis_break_y) {
    # Process x-axis breaks
    xb <- extract_axis_break(object = axis_break_x)
    rng_x <- ggrange2(plot = x, var = "x")
    res_x <- combine_range(xb$breaks, rng_x, xb$scales, xb$ticklabs)
    x_breaks <- res_x$breaks
    x_relrange <- compute_relative_range(breaks = x_breaks, scales = res_x$scales, rng = rng_x)
    if (!rng_x$flagrev %in% c("identity", "reverse")) {
        x_breaks <- lapply(x_breaks, function(i) rng_x$inversefun(i))
    }

    # Process y-axis breaks
    yb <- extract_axis_break(object = axis_break_y)
    rng_y <- ggrange2(plot = x, var = "y")
    res_y <- combine_range(yb$breaks, rng_y, yb$scales, yb$ticklabs)
    y_breaks <- res_y$breaks
    y_relrange <- compute_relative_range(breaks = y_breaks, scales = res_y$scales, rng = rng_y)
    if (!rng_y$flagrev %in% c("identity", "reverse")) {
        y_breaks <- lapply(y_breaks, function(i) rng_y$inversefun(i))
    }

    # Reverse y for top-to-bottom layout (high values at top)
    y_breaks <- rev(y_breaks)
    y_relrange <- rev(y_relrange)

    nx <- length(x_breaks)
    ny <- length(y_breaks)
    margin_x <- xb$space
    margin_y <- yb$space

    # Handle labels
    totallabs <- extract_totallabs(plot = x)
    if (length(totallabs) > 0) {
        x <- .remove_axis_lab(x, totallabs)
    }

    # Handle expand for both axes
    expand_x <- convert_expand(xb$expand)
    expand_y <- convert_expand(yb$expand)
    scaleind_x <- find_scale_index(x, "x")
    scaleind_y <- find_scale_index(x, "y")
    if (!is.null(scaleind_x)) {
        x$scales$scales[[scaleind_x]]$expand <- expand_x
    } else {
        x <- suppressMessages(x + scale_x_continuous(expand = expand_x))
    }
    if (!is.null(scaleind_y)) {
        x$scales$scales[[scaleind_y]]$expand <- expand_y
    } else {
        x <- suppressMessages(x + scale_y_continuous(expand = expand_y))
    }

    legendpos <- check_legend_position(plot = x)

    # Build 2D grid: row-major order (top-to-bottom, left-to-right)
    plots <- vector("list", nx * ny)
    idx <- 0
    for (row_i in seq_len(ny)) {
        row_type <- if (ny == 1) "first"
                    else if (row_i == 1) "last"      # top row
                    else if (row_i == ny) "first"    # bottom row
                    else "other"

        for (col_j in seq_len(nx)) {
            col_type <- if (nx == 1) "first"
                        else if (col_j == 1) "first"     # left column
                        else if (col_j == nx) "last"     # right column
                        else "other"

            idx <- idx + 1
            te <- subplot_theme_2d(
                plot = x,
                col_type = col_type, row_type = row_type,
                margin_x = margin_x, margin_y = margin_y,
                rev_x = rng_x$flagrev, rev_y = rng_y$flagrev,
                symbol_x = xb$symbol, symbol_y = yb$symbol
            )

            plots[[idx]] <- suppressMessages(
                x + coord_cartesian(
                    xlim = c(x_breaks[[col_j]][1], x_breaks[[col_j]][2]),
                    ylim = c(y_breaks[[row_i]][1], y_breaks[[row_i]][2])
                ) + te
            )
        }
    }

    g <- plot_list(
        gglist = setNames(plots, NULL),
        ncol = nx, nrow = ny,
        widths = x_relrange,
        heights = y_relrange,
        guides = "collect",
        output = "patchwork"
    ) & legendpos

    newxlab <- totallabs$x
    newylab <- totallabs$y
    totallabs$x <- NULL
    totallabs$y <- NULL

    g <- ggplotify::as.ggplot(g) + xlab(newxlab) + ylab(newylab)
    g <- set_label(g, totallabs = totallabs, p2 = x)

    # if (!is.null(axis_break_x$symbol) || !is.null(axis_break_y$symbol)) {
    #     g <- add_symbol_annotation(g, axis_break_x, axis_break_y, margin_x, margin_y, rng_x, rng_y, nx, ny, "dual")
    # }

    return(g)
}

#' @export
grid.draw.ggbreak <- function(x, recording = TRUE) {
    class(x) <- class(x)[class(x) != "ggbreak"]
    x <- check_xy_intercept(plot=x)

    axis_break_x <- attr(x, 'axis_break_x')
    axis_break_y <- attr(x, 'axis_break_y')
    has_x <- !is.null(axis_break_x)
    has_y <- !is.null(axis_break_y)

    if (has_x && has_y) {
        g <- render_dual_break(x, axis_break_x, axis_break_y)
        if (recording) {
            print(g)
        }
        return(invisible(g))
    }

    if (has_x) {
        axis_break <- axis_break_x
    } else {
        axis_break <- axis_break_y
    }

    axis_breaks <- extract_axis_break(object=axis_break)
    axis <- axis_breaks$axis
    margin <- axis_breaks$space
    breaks <- axis_breaks$breaks
    expand <- axis_breaks$expand
    scales <- axis_breaks$scales
    ticklabs <- axis_breaks$ticklabs
    rng <- ggrange2(plot=x, var=axis)
    res <- combine_range(breaks, rng, scales, ticklabs)
    breaks <- res$breaks
    scales <- res$scales
    ticklabs <- res$ticklabs

    totallabs <- extract_totallabs(plot=x)
    if (length(totallabs) > 0){
        #x$labels[names(totallabs)] <- NULL
        x <- .remove_axis_lab(x, totallabs)
    }
    nbreaks <- length(breaks)
    subplottheme1 <- subplot_theme(plot=x, axis=axis, type="first", margin = margin, rev = rng$flagrev, symbol = axis_breaks$symbol)
    subplottheme2 <- subplot_theme(plot=x, axis=axis, type="other", margin = margin, rev = rng$flagrev, symbol = axis_breaks$symbol)
    subplottheme3 <- subplot_theme(plot=x, axis=axis, type="last", margin = margin, rev = rng$flagrev, symbol = axis_breaks$symbol)
    coord_fun <- check_coord_flip(plot=x) 
    relrange <- compute_relative_range(breaks=breaks, scales=scales, rng=rng)
    legendpos <- check_legend_position(plot=x)
    if (!rng$flagrev %in% c("identity","reverse")){
        breaks <- lapply(breaks, function(i)rng$inversefun(i))
    }
    scaleind <- find_scale_index(x, axis)
    expand <- convert_expand(expand=expand)
    another.axis = FALSE
    if (!is.null(scaleind)){
        x$scales$scales[[scaleind]]$expand <- expand
        if (!inherits(x$scales$scales[[scaleind]]$name, "waiver")){
            axis.title <- x$scales$scales[[scaleind]]$name
            x <- remove_axis_title(x, axis, coord_fun)
        }else{
            axis.title <- NULL
        }
        if (!inherits(x$scales$scales[[scaleind]]$secondary.axis$name, "waiver")){
            axis.sec.title <- x$scales$scales[[scaleind]]$secondary.axis$name
            x <- remove_axis_title(x, axis, coord_fun, second = TRUE)
        }else{
            axis.sec.title <- NULL
        }
    }else{
        scale_axis <- switch(axis, x=scale_x_continuous, y=scale_y_continuous)
        x <- suppressMessages(x + do.call(scale_axis, list(expand=expand)))
        another_axis <- setdiff(c('x', 'y'), axis)
        another_scaleind <- find_scale_index(x, another_axis)
        if (!is.null(another_scaleind) && !inherits(x$scales$scales[[another_scaleind]]$name, "waiver")){
            axis.title <- x$scales$scales[[another_scaleind]]$name
            x <- remove_axis_title(x, another_axis, coord_fun)
            another.axis = TRUE
        }else{
            axis.title <- NULL
        }
        if (!is.null(another_scaleind) && !inherits(x$scales$scales[[another_scaleind]]$secondary.axis$name, "waiver")){
            axis.sec.title <- x$scales$scales[[another_scaleind]]$secondary.axis$name
            x <- remove_axis_title(x, another_axis, coord_fun, second = TRUE)
            another.axis = TRUE
        }else{
            axis.sec.title <- NULL
        }
    }
    newxlab <- switch(coord_fun, coord_flip=totallabs$y, coord_cartesian=totallabs$x)
    newylab <- switch(coord_fun, coord_flip=totallabs$x, coord_cartesian=totallabs$y)
    if(axis == 'x') {
        p1 <- suppressMessages(x + do.call(coord_fun, list(xlim = c(breaks[[1]][1], breaks[[1]][2]))) + subplottheme1)

        pp1 <- suppressMessages(lapply(breaks[-c(1, nbreaks)], function(i) 
                            x + do.call(coord_fun, list(xlim=c(i[1], i[2]))) + 
                            subplottheme2))
        
        pp2 <- suppressMessages(x + do.call(coord_fun, list(xlim = c(breaks[[nbreaks]][1], breaks[[nbreaks]][2]))) +
               subplottheme3)

        if (length(ticklabs) > 1){
            newticklabs <- ticklabs[-length(ticklabs)]
            for (i in seq_len(length(newticklabs))){
                if (!is.null(scaleind) && !is.null(newticklabs[[i]])){
                    pp1[[i]]$scales$scales[[scaleind]]$breaks <- newticklabs[[i]]
                    pp1[[i]]$scales$scales[[scaleind]]$labels <- newticklabs[[i]]
                }
                if (is.null(scaleind) && !is.null(newticklabs[[i]])){
                    pp1[[i]] <- suppressMessages(pp1[[i]] + scale_x_continuous(breaks=newticklabs[[i]], labels=newticklabs[[i]], expand=expand))
                }
            }
        }

        if (!is.null(scaleind) && !is.null(ticklabs[[length(ticklabs)]]) && rng$flagrev != "reverse"){
            pp2$scales$scales[[scaleind]]$breaks <- ticklabs[[length(ticklabs)]]
            pp2$scales$scales[[scaleind]]$labels <- ticklabs[[length(ticklabs)]]
        }
        if (is.null(scaleind) && !is.null(ticklabs[[length(ticklabs)]]) && rng$flagrev != "reverse"){
            pp2 <- suppressMessages(pp2 + scale_x_continuous(breaks=ticklabs[[length(ticklabs)]], labels=ticklabs[[length(ticklabs)]], expand = expand))
        }
        if (!is.null(scaleind) && !is.null(ticklabs[[length(ticklabs)]]) && rng$flagrev == "reverse"){
            p1$scales$scales[[scaleind]]$breaks <- ticklabs[[length(ticklabs)]]
            p1$scales$scales[[scaleind]]$labels <- ticklabs[[length(ticklabs)]]
        }
        if (is.null(scaleind) && !is.null(ticklabs[[length(ticklabs)]]) && rng$flagrev == "reverse"){
            p1 <- suppressMessages(p1 + scale_x_continuous(breaks = ticklabs[[length(ticklabs)]], labels=ticklabs[[length(ticklabs)]], expand=expand))
            #p1 <- add_expand_new_axis_scale(plot = p1, expand = expand, axis = "x",
            #                                breaks = ticklabs[[length(ticklabs)]], labels=ticklabs[[length(ticklabs)]]
            #      )
        }
        
        g <- switch(coord_fun,
                    coord_flip = plot_list(gglist=setNames(c(list(pp2), rev(pp1), list(p1)), NULL),
                                           ncol=1,
                                           heights=c(rev(relrange[-1]), relrange[1]),
                                           guides = 'collect', output = "patchwork") & legendpos,
                    coord_cartesian = plot_list(gglist=setNames(c(list(p1), pp1, list(pp2)), NULL), 
                                                nrow=1, 
                                                widths=relrange,
                                                guides = 'collect', output = "patchwork") & legendpos
                    )
    } else {
        breaks <- rev(breaks)
        ticklabs <- rev(ticklabs)

        p1 <- suppressMessages(x + do.call(coord_fun, list(ylim = c(breaks[[nbreaks]][1], breaks[[nbreaks]][2]))) + subplottheme1)

        pp1 <- suppressMessages(lapply(breaks[-c(1, nbreaks)], function(i) 
                      x + do.call(coord_fun, list(ylim=c(i[1], i[2]))) +
                            subplottheme2))

        pp2 <- suppressMessages(x + do.call(coord_fun, list(ylim = c(breaks[[1]][1], breaks[[1]][2]))) +
               subplottheme3)
        
        if (length(ticklabs) > 1){
            newticklabs <- ticklabs[-1]
            for (i in seq_len(length(newticklabs))){
                if (!is.null(scaleind) && !is.null(newticklabs[[i]])){
                    pp1[[i]]$scales$scales[[scaleind]]$breaks <- newticklabs[[i]]
                    pp1[[i]]$scales$scales[[scaleind]]$labels <- newticklabs[[i]]
                }
                if (is.null(scaleind) && !is.null(newticklabs[[i]])){
                    pp1[[i]] <- suppressMessages(pp1[[i]] + scale_y_continuous(breaks=newticklabs[[i]], labels=newticklabs[[i]], expand = expand))
                    #pp1[[i]] <- add_expand_new_axis_scale(plot = pp1[[i]], expand = expand, axis= "y",
                    #                                      breaks=newticklabs[[i]], labels=newticklabs[[i]]
                    #            )
                }
            }
        }

        if (!is.null(scaleind) && !is.null(ticklabs[[1]]) && rng$flagrev != "reverse"){
            pp2$scales$scales[[scaleind]]$breaks <- ticklabs[[1]]
            pp2$scales$scales[[scaleind]]$labels <- ticklabs[[1]]
        }
        if (is.null(scaleind) && !is.null(ticklabs[[1]]) && rng$flagrev != "reverse"){
            pp2 <- suppressMessages(pp2 + scale_y_continuous(breaks=ticklabs[[1]], labels=ticklabs[[1]], expand = expand))
            #pp2 <- add_expand_new_axis_scale(plot = pp2, expand = expand, axis = "y",
            #                                 breaks=ticklabs[[1]], labels=ticklabs[[1]])
        }
        if (!is.null(scaleind) && !is.null(ticklabs[[1]]) && rng$flagrev == "reverse"){
            p1$scales$scales[[scaleind]]$breaks <- ticklabs[[1]]
            p1$scales$scales[[scaleind]]$labels <- ticklabs[[1]]
        }
        if (is.null(scaleind) && !is.null(ticklabs[[1]]) && rng$flagrev == "reverse"){
            p1 <- suppressMessages(p1 + scale_y_continuous(breaks=ticklabs[[1]], labels=ticklabs[[1]], expand = expand))
            #p1 <- add_expand_new_axis_scale(plot = p1, expand = expand, axis = "y",
            #                                breaks=ticklabs[[1]], labels=ticklabs[[1]]
            #      )
        }
        
        g <- switch(coord_fun,
                    coord_flip = plot_list(gglist=setNames(c(list(p1), rev(pp1), list(pp2)), NULL), 
                                           nrow=1, 
                                           widths=relrange,
                                           guides = 'collect', output = "patchwork") & legendpos,
                    coord_cartesian = plot_list(gglist=setNames(c(list(pp2), pp1, list(p1)), NULL), 
                                                ncol=1, 
                                                heights=c(rev(relrange[-1]), relrange[1]),
                                                guides = 'collect', output = "patchwork") & legendpos
               )
    }

    totallabs$x <- NULL
    totallabs$y <- NULL
    g <- ggplotify::as.ggplot(g) + xlab(newxlab) + ylab(newylab)
    
    g <- check_axis_title(
            plot = g, 
            axis = axis, 
            coord_fun = coord_fun,
            axis.title = axis.title, 
            axis.sec.title = axis.sec.title,
            another.axis = another.axis
         )

    g <- set_label(g, totallabs = totallabs, p2 = x)

    # if (!is.null(axis_breaks$symbol)) {
    #     g <- add_symbol_annotation(g, axis_breaks, NULL, margin, NULL, rng, NULL, nbreaks + 1, 1, axis)
    # }

    if (recording){
        print(g)
    }
    invisible(g)
}

#' @method grid.draw ggwrap
#' @export
grid.draw.ggwrap <- function(x, recording=TRUE){
    class(x) <- class(x)[class(x) != "ggwrap"]
    x <- check_xy_intercept(plot=x)
    axis_wrap <- attr(x, "axis_wrap")
    totallabs <- extract_totallabs(plot=x)
    if (length(totallabs) > 0){
        #x$labels[names(totallabs)] <- NULL
        x <- .remove_axis_lab(x, totallabs)
    }
    nstep <- axis_wrap$n
    expand <- axis_wrap$expand
    rngrev <- ggrange2(plot=x, 'x')
    rng <- rngrev$axis_range
    if (!is.null(rngrev$flagrev) && rngrev$flagrev == "reverse"){
        rng <- rev(-1 * (rng))
    }
    
    if (!is.null(rngrev$flagrev)) {
        breaks <- seq(rng[1], rng[2], length.out=nstep + 1)
        if (!rngrev$flagrev %in% c("identity", "reverse")){
            breaks <- rngrev$inversefun(breaks)
        }
        x <- add_expand(plot = x, expand = expand, axis = "x")
        gg <- lapply(seq_len(length(breaks)-1), function(i) x + coord_cartesian(xlim=c(breaks[i], breaks[i+1])))
    }else{
        limits <- split_discrete_range(x = rng, n = nstep) 
        gg <- lapply(limits, split_discrete_scale, plot=x, axis='x')
    }
    legendpos <- check_legend_position(plot=x)
    pg <- plot_list(gglist=setNames(gg, NULL), ncol=1, guides="collect", output = "patchwork") & legendpos
    g <- set_label(as.ggplot(pg), totallabs=totallabs, p2=x)
    if (recording){
        print(g)
    }

    invisible(g)

}

#' @method grid.draw ggcut
#' @export
grid.draw.ggcut <- function(x, recording=TRUE){
    class(x) <- class(x)[class(x) != "ggcut"]
    x <- check_xy_intercept(plot=x)
    axis_cut <- attr(x, "axis_cut")
    axis <- axis_cut$axis
    margin <- axis_cut$space
    expand <- axis_cut$expand
    totallabs <- extract_totallabs(plot=x)
    if (length(totallabs) > 0){
        #x$labels[names(totallabs)] <- NULL
        x <- .remove_axis_lab(x, totallabs)
    }
    rngrev <- ggrange2(plot=x, var = axis)
    breaks_relrange <- compute_ggcut_breaks_relrange(ggcut_params=axis_cut, rngrev=rngrev)
    breaks <- breaks_relrange$breaks
    relrange <- breaks_relrange$relrange

    nbreaks <- length(breaks)
    subplottheme1 <- subplot_theme(plot=x, axis=axis, type="first", margin = margin, rev = rngrev$flagrev)
    subplottheme2 <- subplot_theme(plot=x, axis=axis, type="other", margin = margin, rev = rngrev$flagrev)
    subplottheme3 <- subplot_theme(plot=x, axis=axis, type="last", margin = margin, rev = rngrev$flagrev)
    coord_fun <- check_coord_flip(plot=x)
    newxlab <- switch(coord_fun, coord_flip=totallabs$y, coord_cartesian=totallabs$x)
    newylab <- switch(coord_fun, coord_flip=totallabs$x, coord_cartesian=totallabs$y)
    legendpos <- check_legend_position(plot=x)
    if (!rngrev$flagrev %in% c("identity", "reverse")){
        breaks <- rngrev$inversefun(breaks)
    }
    #expand <- getOption(x="scale_xy_expand", default = FALSE)
    x <- add_expand(plot = x, expand = expand, axis = axis)
    if(axis == 'x') {
        p1 <- suppressMessages(x + do.call(coord_fun, list(xlim = c(breaks[[1]][1], breaks[[1]][2]))) + subplottheme1)
        pp1 <- suppressMessages(lapply(breaks[-c(1, nbreaks)], function(i)
                            x + do.call(coord_fun, list(xlim=c(i[1], i[2]))) +
                            subplottheme2))
        pp2 <- suppressMessages(x + do.call(coord_fun, list(xlim = c(breaks[[nbreaks]][1], breaks[[nbreaks]][2]))) +
               subplottheme3)
        g <- switch(coord_fun,
                    coord_flip = plot_list(gglist=setNames(c(list(pp2), rev(pp1), list(p1)), NULL),
                                           ncol=1,
                                           heights=relrange,#c(rev(relrange[-1]), relrange[1]),
                                           guides = 'collect', output = "patchwork") & legendpos, 
                    coord_cartesian = plot_list(gglist=setNames(c(list(p1), pp1, list(pp2)), NULL),
                                                nrow=1,
                                                widths=relrange,
                                                guides = 'collect', output = "patchwork") & legendpos
                    )
    } else {
        breaks <- rev(breaks)
        p1 <- suppressMessages(x + do.call(coord_fun, list(ylim = c(breaks[[nbreaks]][1], breaks[[nbreaks]][2]))) + subplottheme1)
        pp1 <- suppressMessages(lapply(breaks[-c(1, nbreaks)], function(i)
                      x + do.call(coord_fun, list(ylim=c(i[1], i[2]))) +
                            subplottheme2))
        pp2 <- suppressMessages(x + do.call(coord_fun, list(ylim = c(breaks[[1]][1], breaks[[1]][2]))) +
               subplottheme3)
        g <- switch(coord_fun,
                    coord_flip = plot_list(gglist=setNames(c(list(p1), rev(pp1), list(pp2)), NULL),
                                           nrow=1,
                                           widths=relrange,
                                           guides = 'collect', output = "patchwork") & legendpos,
                    coord_cartesian = plot_list(gglist=setNames(c(list(pp2), pp1, list(p1)), NULL),
                                                ncol=1,
                                                heights=relrange,#c(rev(relrange[-1]), relrange[1]),
                                                guides = 'collect', output = "patchwork") & legendpos
               )
    }
    totallabs$x <- NULL
    totallabs$y <- NULL
    g <- ggplotify::as.ggplot(g) + xlab(newxlab) + ylab(newylab)
    g <- set_label(g, totallabs = totallabs, p2 = x)
    if (recording){
        print(g)
    }
    
    invisible(g)

}

#' Set an axis break point for a 'gg' plot
#'
#' This scale function set an axis break point for a 'gg' plot. Either 'x' and 'y' axes are supported. The result is still a 'gg' object and user can progressively add layers to it. 
#' @title scale_x_break
#' @rdname scale_break
#' @param breaks break point
#' @param scales relative width or height of subplots,
#' default is "fixed". If scale is 'free', all subplots have 
#' equal width or height. It also can be any number to set 
#' relative width or height compare to first subplot.
#' @param ticklabels the axis labels to subplot, default is NULL.
#' @param expand default is TRUE, logical or a vector of range expansion constants
#' used to add some padding around the data to ensure that they
#' are placed some distance away from the axes. Use the convenience 
#' function \code{expansion()} of \code{ggplot2} to generate the values for
#' the \code{expand} argument. The defaults are to expand the scale
#' by 5% on each side for continuous variables. If it is logical, the \code{TRUE} means
#' the default of \code{ggplot2} (foregoing statement), and \code{FALSE} means no expand for the plot.
#' @param space the blank space among the subplots after break, default is 0.1 (cm).
#' @param symbol the break symbol, default is NULL, could be 'slash'.
#' @return gg object
#' @export
#' @author Guangchuang Yu
scale_x_break <- function(breaks, scales="fixed", ticklabels=NULL, expand=TRUE, space = 0.1, symbol = NULL) {
    .ggbreak$scale_break('x', breaks, scales, ticklabels, expand, space, symbol)
}

#' @title scale_y_break
#' @rdname scale_break
#' @export
#' @import yulab.utils
#' @examples
#' require(ggplot2 )
#' set.seed(2019-01-19)
#' d <- data.frame(
#'   x = 1:20,
#'   y = c(rnorm(5) + 4, rnorm(5) + 20, rnorm(5) + 5, rnorm(5) + 22)
#' )
#'
#' p <- ggplot(d, aes(x, y)) + geom_col()
#' x <- p+scale_y_break(c(7, 17 ) )
#' print(x)
scale_y_break <- function(breaks, scales="fixed", ticklabels=NULL, expand=TRUE, space = .1, symbol = NULL) {
    .ggbreak$scale_break('y', breaks, scales, ticklabels, expand, space, symbol)
}



#' This scale function wraps a 'gg' plot over multiple rows to make plots with long x axes easier to read.
#'
#' 
#' @title scale-wrap
#' @rdname scale-wrap
#' @param n the number of subplot pieces.
#' @return gg object
#' @export
#' @examples
#' library(ggplot2)
#' library(ggbreak)
#' p <- ggplot(economics, aes(x=date, y = unemploy, colour = uempmed)) +
#'      geom_line()
#' p + scale_wrap(n=4)
scale_wrap <- function(n){
    structure(list(n = n, expand = FALSE), 
              class = "wrap_params")
}


#' @title scale_x_cut
#' @param breaks a numeric or numeric vector, the points to be divided
#' @param which integer, the position of subplots to scales, started from left to right or top to bottom.
#' @param scales numeric, relative width or height of subplots.
#' @param expand default is FALSE, logical a vector of range expansion constants
#' used to add some padding around the data to ensure that they
#' are placed some distance away from the axes. Use the convenience
#' function \code{expansion()} of \code{ggplot2} to generate the values for
#' the \code{expand} argument. The defaults are to expand the scale
#' by 5% on each side for continuous variables. If it is logical, the \code{TRUE} means
#' the default of \code{ggplot2} (foregoing statement), and \code{FALSE} means no expand for the plot.
#' @param space the blank space among the subplots after cut, default is 0.1 (cm). 
#' @rdname scale_cut
#' @return gg object
#' @export
scale_x_cut <- function(breaks, which=NULL, scales=NULL, expand = FALSE, space = .1){
    scale_cut(
        axis = "x",
        breaks = breaks,
        which = which,
        scales = scales,
        expand = expand,
        space = space
    )
}

#' @title scale_y_cut
#' @rdname scale_cut
#' @export
#' @examples
#' library(ggplot2)
#' library(ggbreak)
#' set.seed(2019-01-19)
#' d <- data.frame(
#'      x = 1:20,
#'      y = c(rnorm(5) + 4, rnorm(5) + 20, rnorm(5) + 5, rnorm(5) + 22)
#'  )
#' p <- ggplot(d, aes(x, y)) + geom_col()
#' p + scale_y_cut(breaks=c(7, 18), which=c(1, 3), scales=c(3, 0.5))
scale_y_cut <- function(breaks, which=NULL, scales=NULL, expand = FALSE, space = .1){
    scale_cut(
        axis = "y",
        breaks = breaks,
        which = which,
        scales = scales,
        expand = expand,
        space = space
    )

}

scale_cut <- function(axis, breaks, which=NULL, scales=NULL, expand = FALSE, space = .1){
    structure(list(axis = axis, 
                   breaks = breaks, 
                   which = which, 
                   scales = scales,
                   expand = expand,
                   space = space
                   ),
    class = "ggcut_params")
}

library(ggplot2)
library(ggbreak)

test_that("Regression: Single x/y breaks render correctly", {
  p1 <- ggplot(mpg, aes(displ, hwy)) + geom_point() + scale_x_break(c(3, 4))
  expect_error(print(p1), NA)
  
  p2 <- ggplot(mpg, aes(displ, hwy)) + geom_point() + scale_y_break(c(25, 35))
  expect_error(print(p2), NA)
  
  p3 <- ggplot(mpg, aes(displ, hwy)) + geom_point() +
    scale_x_break(c(3, 4)) + scale_x_break(c(5, 6))
  expect_error(print(p3), NA)
})

test_that("Dual axis rendering: Basic x + y", {
  p4 <- ggplot(mpg, aes(displ, hwy)) + geom_point() +
    scale_x_break(c(3, 4)) + scale_y_break(c(25, 35))
  expect_error(print(p4), NA)
})

test_that("Dual axis rendering: y + x order", {
  p5 <- ggplot(mpg, aes(displ, hwy)) + geom_point() +
    scale_y_break(c(25, 35)) + scale_x_break(c(3, 4))
  expect_error(print(p5), NA)
})

test_that("Dual axis rendering: Scatter with outlier clusters", {
  set.seed(42)
  df <- data.frame(
    x = c(rnorm(50, 5, 1), rnorm(10, 50, 2)),
    y = c(rnorm(50, 10, 2), rnorm(10, 100, 5))
  )
  p6 <- ggplot(df, aes(x, y)) + geom_point() +
    scale_x_break(c(10, 45)) + scale_y_break(c(20, 90))
  expect_error(print(p6), NA)
})

test_that("Dual axis rendering: With title and theme", {
  p7 <- ggplot(mpg, aes(displ, hwy)) +
    geom_point(aes(color = class)) +
    scale_x_break(c(3, 4)) + scale_y_break(c(25, 35)) +
    labs(title = "Dual axis break", subtitle = "with color") +
    theme_minimal()
  expect_error(print(p7), NA)
})

test_that("Dual axis rendering: Parameters (space, scales)", {
  set.seed(42)
  df <- data.frame(
    x = c(rnorm(50, 5, 1), rnorm(10, 50, 2)),
    y = c(rnorm(50, 10, 2), rnorm(10, 100, 5))
  )
  
  # Test space parameter
  p8 <- ggplot(df, aes(x, y)) + geom_point() +
    scale_x_break(c(10, 45), space = 0.5) +
    scale_y_break(c(20, 90), space = 0.5)
  expect_error(print(p8), NA)
  
  # Test scales parameter
  p9 <- ggplot(df, aes(x, y)) + geom_point() +
    scale_x_break(c(10, 45), scales = 0.3) +
    scale_y_break(c(20, 90), scales = 0.3)
  expect_error(print(p9), NA)
})

test_that("Dual axis rendering: Multiple breaks on BOTH axes", {
  set.seed(42)
  df <- data.frame(
    x = c(rnorm(50, 5, 1), rnorm(10, 50, 2)),
    y = c(rnorm(50, 10, 2), rnorm(10, 100, 5))
  )
  p10 <- ggplot(df, aes(x, y)) + geom_point() +
    scale_x_break(c(10, 20)) + scale_x_break(c(30, 45)) +
    scale_y_break(c(20, 40)) + scale_y_break(c(60, 90))
  expect_error(print(p10), NA)
})

test_that("Dual axis rendering: Compatibility with various themes", {
  base_p <- ggplot(mpg, aes(displ, hwy)) +
    geom_point(aes(color = class)) +
    scale_x_break(c(3, 4)) + scale_y_break(c(25, 35)) +
    labs(title = "Dual axis break")
  
  expect_error(print(base_p + theme_bw()), NA)
  expect_error(print(base_p + theme_linedraw()), NA)
  expect_error(print(base_p + theme_classic()), NA)
})

library(ggplot2)
library(ggbreak)

test_that("scale_x_break with symbol renders correctly", {
  p <- ggplot(mpg, aes(displ, hwy)) + 
    geom_point() +
    scale_x_break(c(3, 4), symbol = "slash") +
    theme_bw()
  
  # Just verify it builds without error
  expect_error(print(p), NA)
})

test_that("scale_y_break with symbol renders correctly", {
  p <- ggplot(mpg, aes(displ, hwy)) + 
    geom_point() +
    scale_y_break(c(25, 30), symbol = "slash") +
    theme_linedraw()
  
  expect_error(print(p), NA)
})

test_that("Dual break with symbols renders correctly", {
  p <- ggplot(mpg, aes(displ, hwy)) + 
    geom_point() +
    scale_x_break(c(3, 4), symbol = "slash") +
    scale_y_break(c(25, 30), symbol = "slash") +
    theme_bw()
  
  expect_error(print(p), NA)
})

library(ggplot2)
library(ggbreak)

test_that("Single axis break attributes are stored correctly", {
  p1 <- ggplot(mpg, aes(displ, hwy)) + geom_point() + scale_x_break(c(3, 4))
  expect_false(is.null(attr(p1, "axis_break_x")))
  expect_true(is.null(attr(p1, "axis_break_y")))
  
  p2 <- ggplot(mpg, aes(displ, hwy)) + geom_point() + scale_y_break(c(25, 35))
  expect_false(is.null(attr(p2, "axis_break_y")))
  expect_true(is.null(attr(p2, "axis_break_x")))
})

test_that("Multiple breaks on same axis are stored correctly", {
  p3 <- ggplot(mpg, aes(displ, hwy)) + geom_point() +
    scale_x_break(c(3, 4)) + scale_x_break(c(5, 6))
  ab3 <- attr(p3, "axis_break_x")
  expect_s3_class(ab3, "ggbreak_params")
  # Depending on implementation, it might be a list of params or a specific class
  # Based on ggbreak implementation, multiple breaks might be stored in a list structure
  # But here we just verify it exists.
  expect_false(is.null(ab3))
})

test_that("Dual axis breaks are stored correctly (x + y)", {
  p4 <- ggplot(mpg, aes(displ, hwy)) + geom_point() +
        scale_x_break(c(3, 4)) + scale_y_break(c(25, 35))
  
  expect_false(is.null(attr(p4, "axis_break_x")))
  expect_false(is.null(attr(p4, "axis_break_y")))
  
  # Check if it renders
  expect_error(print(p4), NA)
})

test_that("Dual axis breaks are stored correctly (y + x)", {
  p5 <- ggplot(mpg, aes(displ, hwy)) + geom_point() +
        scale_y_break(c(25, 35)) + scale_x_break(c(3, 4))
  
  expect_false(is.null(attr(p5, "axis_break_x")))
  expect_false(is.null(attr(p5, "axis_break_y")))
})

test_that("Layers can be added to dual-axis break plot", {
  p6 <- ggplot(mpg, aes(displ, hwy)) + geom_point() +
        scale_x_break(c(3, 4)) + scale_y_break(c(25, 35)) +
        labs(title = "Dual break") + theme_minimal()
  
  expect_false(is.null(attr(p6, "axis_break_x")))
  expect_false(is.null(attr(p6, "axis_break_y")))
  expect_error(print(p6), NA)
})

#' @importFrom ggplot2 ggplot_add
#' @importFrom rlang abort
#' @method ggplot_add ggbreak_params
#' @export
ggplot_add.ggbreak_params <- function(object, plot, ...) {
    axis <- object$axis
    attr_name <- paste0('axis_break_', axis)

    if (inherits(plot, "ggbreak")) {
        existing <- attr(plot, attr_name)
        if (!is.null(existing)) {
            # Same axis: merge multiple breaks
            object <- list.add(existing, object)
        }
    }

    attr(plot, attr_name) <- object
    if (!"ggbreak" %in% class(plot)) {
        class(plot) <- c("ggbreak", class(plot))
    }

    return(plot)
}


#' @method ggplot_add wrap_params
#' @export
ggplot_add.wrap_params <- function(object, plot, ...){
    attr(plot, "axis_wrap") <- object
    class(plot) <- c("ggwrap", class(plot))
    return(plot)
}


#' @method ggplot_add ggcut_params
#' @export
ggplot_add.ggcut_params <- function(object, plot, ...){
    attr(plot, "axis_cut") <- object
    class(plot) <- c("ggcut", class(plot))
    return (plot)
}

#' @method ggplot_add ggbreak
#' @importFrom ggfun is.ggbreak
#' @export
ggplot_add.ggbreak <- function(object, plot, ...) {
    if (is.ggbreak(plot)) {
        ggplot_add(ggbreak2ggplot(object),
                   ggbreak2ggplot(plot),
                   ...)
    } else{
        ggplot_add(as.ggplot(grid.draw(object, recording=FALSE)),
                   as.ggplot(plot),
                   ...)
    }
}


#' @method ggplot_add ggwrap
#' @export
ggplot_add.ggwrap <- ggplot_add.ggbreak

#' @method ggplot_add ggcut
#' @export
ggplot_add.ggcut <- ggplot_add.ggbreak

#' @method ggplot_add gg
#' @importFrom ggfun ggbreak2ggplot
#' @export
ggplot_add.gg <- function(object, plot, ...){
    if (is.ggbreak(plot)){
        if (inherits(object, all_class_gg)){
            tmp <- class(plot)
            plot <- .drop_class(plot, "ggbreak")
            plot <- ggplot_add(object, plot, ...)
            class(plot) <- tmp
            return(plot)
        }else{
            ggplot_add(as.ggplot(object), ggbreak2ggplot(plot), ...)
	}
    } else{
        NextMethod()
    }
}


all_class_gg <- c("Scale", "Guides", "Coord", "Facet", "Layer", 
                  "Layout", "theme", "ggplot2::labels", "ggplot2::mapping")

## function taken from ggtree
#' @importFrom ggplot2 ggplot_build
ggrange2 <- function (plot, var) {
    var <- paste0("panel_scales_", var)
    gb <- ggplot_build(plot)
    limits <- gb$layout[[var]][[1]]$limits
    if (!is.null(limits)){
        axis_range <- limits
    }else{
        axis_range <- gb$layout[[var]][[1]]$range$range
    }
    flagrev <- gb$layout[[var]][[1]]$trans$name
    transfun <- gb$layout[[var]][[1]]$trans$transform
    inversefun <- gb$layout[[var]][[1]]$trans$inverse
    list(axis_range=axis_range, flagrev=flagrev, transfun=transfun, inversefun=inversefun)
}

check_legend_position <- function(plot){
    if (!is.null(plot$theme$legend.position)){
        tm <- theme(legend.position=plot$theme$legend.position)
    }else{
        tm <- theme()
    }
    return (tm)
}

#' @importFrom ggplot2 labs
set_label <- function(p, totallabs, p2 = NULL) {
    p <- p + 
         do.call(labs, totallabs) 

    if (is.null(p2)) {
        has_theme <- FALSE
    } else {
        has_theme <- length(p2$theme) != 0
    }

    if (has_theme) {
        x <- p2 
    } else {
        x <- NULL
    }
    labs_params <- c("text", "title", "axis.title", 
                     "axis.title.x", "axis.title.x.top", "axis.title.x.bottom",
                     "axis.title.y", "axis.title.y.left", "axis.title.y.right",
                     "plot.title", "plot.title.position", "plot.subtitle",
                     "plot.caption", "plot.caption.position", "plot.tag", "plot.tag.position")
    p <- p + 
         theme_fp(x=x, i=labs_params) +
         theme(axis.text = element_blank(),
               axis.ticks = element_blank()
               )
    return(p)
}

extract_totallabs <- function(plot){
    alllabs <- ggplot_build(plot)$plot$labels
    totallabs <- alllabs[names(alllabs) %in% c("x", "y", "title", "subtitle", "caption", "tag")]
    totallabs
}


combine_range <- function(breaks, rangeres, scales, ticklabs){
    if (rangeres$flagrev=="reverse"){
        rangeres$axis_range <- rev(-1 * (rangeres$axis_range))
    }
    if (rangeres$flagrev=="date"){
        if (is.list(breaks)){
            breaks <- lapply(breaks, function(i) as.Date(i))
        }else{
            breaks <- as.Date(breaks)
        }
    }
    if (!rangeres$flagrev %in% c("identity", "reverse")){
        if (is.list(breaks)){
            breaks <- lapply(breaks, function(i) rangeres$transfun(i))
        }else{
            breaks <- rangeres$transfun(breaks)
        }
    }
    res <- merge_intervals(breaks, scales, ticklabs)
    newbreaks <- res$breaks
    newscales <- res$scales
    newticklabs <- res$ticklabs
    newbreaks <- c(rangeres$axis_range[1], unlist(newbreaks), rangeres$axis_range[2])
    newbreaks <- lapply(data.frame(matrix(newbreaks, nrow=2)), function(i)i)
    if (rangeres$flagrev=="reverse"){
        newbreaks <- lapply(newbreaks, function(i)rev(i))
        return(list(breaks=rev(newbreaks), scales=rev(newscales), 
                    ticklabs=c(rev(newticklabs[-length(newticklabs)]), newticklabs[length(newticklabs)])))
    }
    return(list(breaks=newbreaks, scales=newscales, ticklabs=newticklabs))
}

merge_intervals <- function(breaks, scales, ticklabs){
    if (!inherits(breaks, "list")){
        breaks <- list(breaks)
    }
    if (!inherits(ticklabs, "list")){
        ticklabs <- list(ticklabs)
    }
    newbreaks <- list()
    newscales <- list()
    newticklabs <- list()
    breaks <- lapply(breaks, function(i) sort(i))
    ind <- order(unlist(lapply(breaks, function(i)i[1])))
    scales <- scales[ind]
    breaks <- breaks[ind]
    ticklabs <- ticklabs[ind]
    for (i in seq_len(length(breaks))){
        if (length(newbreaks) >= 1 && breaks[[i]][1] <= newbreaks[[length(newbreaks)]][2]){
            newbreaks[[length(newbreaks)]][2] <- max(newbreaks[[length(newbreaks)]][2], breaks[[i]][2])
            mergescales <- c(scales[[i]], newscales[[length(newscales)]])
            mergeticks <- c(list(ticklabs[[i]], list(newticklabs[[length(newticklabs)]])))
            if (any("fixed" %in% mergescales)){
                newscales[[length(newscales)]] <- "fixed"
            }
            if ((!"fixed" %in% mergescales) && any("free" %in% mergescales)){
                newscales[[length(newscales)]] <- "free"
            }
            if (is.numeric(mergescales)){
                newscales[[length(newscales)]] <- max(mergescales)
            }
            newticklabs[[length(newticklabs)]] <- mergeticks[[which.max(unlist(lapply(mergeticks, function(i)length(i))))]]
        }else{
            newbreaks <- c(newbreaks, list(breaks[[i]]))
            newscales <- c(newscales, list(scales[[i]]))
            newticklabs <- c(newticklabs, list(ticklabs[[i]]))
        }
    }
    return(list(breaks=newbreaks, scales=unlist(newscales), ticklabs=newticklabs))
}

extract_axis_break <- function(object){
    if (inherits(object, "ggbreak_params")){
        axis <- object$axis
        breaks <- object$breaks
        expand <- object$expand
        scales <- object$scales
        ticklabs <- object$ticklabels
        space <- object$space
        symbol <- object$symbol
    }else{
        axis <- object[[1]]$axis
        space <- object[[1]]$space
        expand <- object[[length(object)]]$expand
        breaks <- lapply(object, function(i)i$breaks)
        scales <- lapply(object, function(i)i$scales)
        ticklabs <- lapply(object, function(i)i$ticklabels)
        symbol <- object[[1]]$symbol
    }
    return(list(
                axis = axis,
                space = space,
                breaks = breaks,
                expand = expand,
                scales = scales,
                ticklabs = ticklabs,
                symbol = symbol
            ))
}

compute_ggcut_breaks_relrange <- function(ggcut_params, rngrev){
    if (rngrev$flagrev == "reverse"){
        rngrev$axis_range <- rev(-1 * (rngrev$axis_range))
    }
    breaks <- ggcut_params$breaks
    if (rngrev$flagrev == "date"){
        breaks <- as.Date(breaks)
    }
    if (!rngrev$flagrev %in% c("identity", "reverse")){
        breaks <- rngrev$transfun(breaks)
    }
    if (any(breaks < rngrev$axis_range[1]) || any(breaks > rngrev$axis_range[2])){
        abort("Some breaks are not in the plot range. Please check all breaks!")
    }
    if (length(breaks) > 1){
        breaks <- c(rngrev$axis_range[1], sort(breaks), rngrev$axis_range[2])
        breaks <- lapply(seq_len(length(breaks)-1), function(i) c(breaks[i], breaks[i+1]))
    }else{
        breaks <- list(c(rngrev$axis_range[1], breaks), 
                       c(breaks, rngrev$axis_range[2]))
    }
    relrange <- rep(1, length(breaks))
    if (!is.null(ggcut_params$which) && !is.null(ggcut_params$scales)){
        relrange[ggcut_params$which] <- ggcut_params$scales
    }
    if (rngrev$flagrev == "reverse"){
        breaks <- rev(lapply(breaks, function(i) rev(i)))
        #relrange <- rev(relrange)
    }
    return(list(breaks=breaks, relrange=relrange))
}


theme_no_margin <- getFromNamespace("theme_no_margin", "ggfun")
theme_fp <- getFromNamespace('theme_fp', 'ggfun')


list.add <- function(obj, ...){
    if (inherits(obj, "ggbreak_params")){
        c(list(obj), list(...))
    }else{
        c(obj, list(...))
    }
}

remove_axis_title <- function(plot, axis, coord_fun, second = FALSE){
    axis <- switch(coord_fun,
                   coord_flip = setdiff(c("x", "y"), axis),
                   coord_cartesian = intersect(c("x", "y"), axis))    
    if (axis == "x"){
        if (second){
            plot <- plot + ggplot2::guides(x.sec = ggplot2::guide_axis(title = NULL))
        }else{
            plot <- plot + ggplot2::guides(x = ggplot2::guide_axis(title = NULL))
        }
    }else if (axis == "y"){
        if (second){
            plot <- plot + ggplot2::guides(y.sec = ggplot2::guide_axis(title = NULL))
        }else{
            plot <- plot + ggplot2::guides(y = ggplot2::guide_axis(title = NULL))
        }
    }
    return(plot)
}

check_axis_title <- function(plot, axis, coord_fun, axis.title, axis.sec.title, another.axis){
    if (another.axis){
        axis <- setdiff(c('x','y'), axis)
    }
        
    axis <- switch(coord_fun, 
                   coord_flip = setdiff(c("x", "y"), axis), 
                   coord_cartesian = axis)
    if (!is.null(axis.sec.title)){
        if (axis == "x"){
            plot <- plot + ggplot2::guides(x.sec = ggplot2::guide_axis(title=axis.sec.title))
        }
        if (axis == "y"){
            plot <- plot + ggplot2::guides(y.sec = ggplot2::guide_axis(title=axis.sec.title))
        }
    }
    if (!is.null(axis.title)){
        if (axis == "x"){
            plot <- plot + ggplot2::guides(x = ggplot2::guide_axis(title=axis.title))
        }
        if (axis == "y"){
            plot <- plot + ggplot2::guides(y = ggplot2::guide_axis(title=axis.title))
        }
    }
    return (plot)
}

find_scale_index <- function(plot, aesthetic){
    if (plot$scales$has_scale(aesthetic)){
        scaleind <- which(plot$scales$find(aesthetic))
    }else{
        scaleind <- NULL
    }
    return(scaleind)
}


split_discrete_range <- function(x, n){
    if (n >= 2){
        l <- max(table(cut(seq_along(x), n, label=FALSE)))
    }else{
        l <- length(x)
    }
    x <- split(x, ceiling(seq_along(x)/l))
    return(x)
}

split_discrete_scale <- function(limits, plot, axis='x'){
   var <- paste0("panel_scales_", axis)
   gb <- ggplot_build(plot)
   scales_axis_obj <- gb$layout[[var]][[1]]
   scales_axis_obj$limits <- limits
   plot <- plot + scales_axis_obj
   return(plot)
}

numeric2Date <- function(x) {
    as.Date(x, origin="1970-01-01")
}


.remove_axis_lab <- function(x, labels){
   params <- lapply(seq(length(labels)), function(i)NULL)
   names(params) <- names(labels)
   x + do.call('labs', params)
}

.drop_class <- function(x, class){
    old <- class(x)
    class(x) <- old[!old %in% class]
    return(x)
}

#' @method plot ggbreak
#' @export
plot.ggbreak <- function(x, y, ...) {
    NextMethod()
}

#' @method plot ggwrap
plot.ggwrap <- function(x, y, ...){
    NextMethod()
}

#' @method plot ggcut
plot.ggcut <- function(x, y, ...){
    NextMethod()
}

#' @method print ggbreak
#' @export
print.ggbreak <- function(x,...) {
    suppressWarnings(grid.draw.ggbreak(x, ...))
}

#' @method print ggwrap
#' @export
print.ggwrap <- function(x, ...){
    suppressWarnings(grid.draw.ggwrap(x, ...))
}

#' @method print ggcut
#' @export
print.ggcut <- function(x, ...){
    suppressWarnings(grid.draw.ggcut(x, ...))
}

subplot_theme <- function(plot, axis, type, margin = .2, rev, symbol = NULL){
    type <- check_strip_pos(plot=plot, type=type)
    axis.pos <- check_another_position(plot = plot, axis = axis)
    axis <- check_theme_coflip(plot=plot, axis=axis)
    if (axis.pos == 'another.position'){
        te <- switch(type,
                     first = axis_theme(plot=plot, axis = axis) +
                             strip_theme(plot=plot, axis=axis) +
                             first_margin_theme(axis = axis, margin = margin, rev=rev),
                     other = axis_theme(plot=plot, axis=axis) +
                             strip_theme(plot, axis=axis) +
                             other_margin_theme(axis = axis, margin = margin),
                     last = last_margin_theme(axis = axis, margin = margin, rev=rev),
                     #internalfirst = axis_theme(plot=plot, axis=axis) +
                     #    strip_theme(plot=plot, axis=axis),
                     internallast = list())        
    }else if (axis.pos == 'another.secondary.axis'){
        te <- switch(type,
                     first = axis_theme(plot = plot, axis = axis, secondary.axis = TRUE) +
                             strip_theme(plot=plot, axis=axis) + 
                             first_margin_theme(axis = axis, margin = margin, rev=rev),
                     other = axis_theme(plot=plot, axis=axis, secondary.axis=TRUE, type = 'other') +
                             strip_theme(plot, axis=axis) +
                             other_margin_theme(axis = axis, margin = margin),
                     last =  axis_theme(plot=plot, axis = axis, secondary.axis = TRUE, type='last') +
                             last_margin_theme(axis = axis, margin = margin, rev=rev),
        
              )
    
    }else{
        te <- switch(type,
                     first = strip_theme(plot=plot, axis=axis) +
                             first_margin_theme(axis = axis, margin = margin, rev=rev),
                     other = axis_theme(plot=plot, axis=axis) + 
                             strip_theme(plot, axis=axis) +
                             other_margin_theme(axis = axis, margin = margin),
                     last = axis_theme(plot=plot, axis = axis) +
                            last_margin_theme(axis = axis, margin = margin, rev=rev),
                     #internalfirst = axis_theme(plot=plot, axis=axis) + 
                     #    strip_theme(plot=plot, axis=axis),
                     internallast = list())
    } 
    #te <- te + theme_no_margin()
    if (!is.null(symbol)) {
        border_te <- .symbol_theme(te, plot, axis, type, symbol, rev)
        if (!is.null(border_te)) {
            te <- te + border_te
        }
    }
    return(te)
}

.symbol_theme <- function(te, plot, axis, type, symbol, rev) {
    full_theme <- ggplot2::theme_get() + plot$theme + te
    border <- full_theme$panel.border
    
    # We should only draw symbols on the edges that actually have a border or axis line.
    # To keep it robust, we can use the `sides` parameter to specifically target `top`, `bottom`, `left`, `right`.
    # For 1D break, if it's an X-axis break, we typically only want symbols on the bottom (or top if there's an axis/border there).
    # Since `ggbreak` handles top/bottom via `first`/`last`/`other` for strips and axes,
    # let's assume we want to draw symbols on the broken edges.
    
    sides <- character(0)
    if (axis == "x") {
        # The break gap is vertical, so the symbols should be drawn on the top and bottom borders.
        # But we only want them on the side where the break happens.
        # Wait, if X is broken, the left panel's right edge is broken, and the right panel's left edge is broken.
        # So we want symbols on the top-right & bottom-right of the left panel,
        # and top-left & bottom-left of the right panel.
        if (type == "first") {
            sides <- if (rev == "reverse") "left" else "right"
        } else if (type == "last") {
            sides <- if (rev == "reverse") "right" else "left"
        } else if (type == "other") {
            sides <- c("left", "right")
        }
    } else if (axis == "y") {
        # The break gap is horizontal.
        if (type == "first") {
            sides <- if (rev == "reverse") "bottom" else "top"
        } else if (type == "last") {
            sides <- if (rev == "reverse") "top" else "bottom"
        } else if (type == "other") {
            sides <- c("top", "bottom")
        }
    }
    
    if (length(sides) > 0) {
        # By default, we will just pass these sides. 
        # But `element_break_symbol` should be smart enough to only draw where there is an actual border.
        # Actually, let's just pass `axis` so the grob knows if it's an X or Y break.
        return(theme(panel.border = element_break_symbol(symbol = symbol, sides = sides, base_border = border, axis = axis)))
    }
    return(NULL)
}

axis_theme <- function(plot, axis, secondary.axis=FALSE, type='first'){
    if (secondary.axis){
        if (type == 'first'){
            axis_theme <- switch(axis,
                                 x = theme(axis.text.y.right = element_blank(),
                                           axis.ticks.y.right = element_blank(),
                                           axis.line.y.right = element_blank(),
                                           axis.title.y.right = element_blank()
                                     ),
                                 y = theme(axis.text.x.top = element_blank(),
                                           axis.ticks.x.top = element_blank(),
                                           axis.line.x.top = element_blank(),
                                           axis.title.x.top = element_blank()
                                 )
            
                          )    
        }else if (type == 'last'){
            axis_theme <- switch(axis,
                                 x = theme(axis.text.y.left = element_blank(),
                                           axis.ticks.y.left = element_blank(),
                                           axis.line.y.left = element_blank(),
                                           axis.title.y.left = element_blank()
                                     ),
                                 y = theme(axis.text.x.bottom = element_blank(),
                                           axis.ticks.x.bottom = element_blank(),
                                           axis.line.x.bottom = element_blank(),
                                           axis.title.x.bottom = element_blank()
                                 )

                          ) 
        
        }else{
        axis_theme <- switch(axis,
                             x = theme(axis.text.y=element_blank(),
                                      axis.ticks.y=element_blank(),
                                      axis.line.y=element_blank(),
                                      axis.title.y=element_blank()),
                             y = theme(axis.text.x=element_blank(),
                                      axis.ticks.x=element_blank(),
                                      axis.line.x=element_blank(),
                                      axis.title.x=element_blank())

                      )        
        
        }
    }else{
        axis_theme <- switch(axis, 
                            x = theme(axis.text.y=element_blank(),
                                      axis.ticks.y=element_blank(),
                                      axis.line.y=element_blank()),
                            y = theme(axis.text.x=element_blank(),
                                      axis.ticks.x=element_blank(),
                                      axis.line.x=element_blank())
                      )
    }
    return(axis_theme)
}

strip_theme <- function(plot, axis){
    sp_theme <- switch(axis, 
                      x = theme(strip.background.y=element_blank(),
                                strip.text.y=element_blank()),
                      y = theme(strip.background.x=element_blank(),
                                strip.text.x=element_blank())
                )
    return(sp_theme)
}

#' @importFrom ggplot2 margin
first_margin_theme <- function(axis, margin, rev){
    if (rev == "reverse"){
      fmg_theme <- switch(axis,
        x = theme(plot.margin = margin(l = margin/2, unit = "cm")),
        y = theme(plot.margin = margin(b = margin/2, unit = "cm")),
      )
    }else{
      fmg_theme <- switch(axis,
        x = theme(plot.margin = margin(r = margin/2, unit = "cm")),
        y = theme(plot.margin = margin(t = margin/2, unit = "cm"))
      )
    }
    return (fmg_theme)
}

other_margin_theme <- function(axis, margin){
    mg_theme <- switch(axis,
      x = theme(plot.margin = margin(r = margin/2, l = margin/2, unit = "cm")),
      y = theme(plot.margin = margin(t = margin/2, b = margin/2, unit = "cm"))
    )
}

last_margin_theme <- function(axis, margin, rev){
    if (rev == "reverse"){
      lmg_theme <- switch(axis,
        x = theme(plot.margin = margin(r=margin/2, unit = "cm")),
        y = theme(plot.margin = margin(t=margin/2, unit = "cm"))
      )
    }else{
      lmg_theme <- switch(axis,
        x = theme(plot.margin = margin(l = margin/2, unit = "cm")),
        y = theme(plot.margin = margin(b = margin/2, unit = "cm"))
      )
    }
}

check_strip_pos <- function(plot, type){
    if (length(plot$facet$params)>0){
        flagwrap <- plot$facet$params$strip.position
        if ((!is.null(flagwrap) && flagwrap %in% c("bottom", "left")) || 
            !is.null(plot$facet$params$switch)){
            type <- switch(type,
                           first = "internallast",
                           last = "other",
                           other = "other"
                          )
        }
    }
    return(type)
}

check_theme_coflip <- function(plot, axis){
    if (inherits(plot, "gg") && inherits(plot$coordinates, "CoordFlip")){
        axis <- switch(axis, x = "y", y = "x" )
    }
    return(axis)
}

check_another_position <- function(plot, axis){
    another.axis <- setdiff(c("x", "y"), axis)
    if (plot$scales$has_scale(another.axis)){
        another.scaleind <- which(plot$scales$find(another.axis)) 
        if (#!inherits(plot$scales$scales[[another.scaleind]]$secondary.axis, "waiver") ||
            (another.axis == "x" && plot$scales$scales[[another.scaleind]]$position !='bottom') ||
            (another.axis == 'y' && plot$scales$scales[[another.scaleind]]$position !='left')
        ){
            return('another.position')
        }else if (!inherits(plot$scales$scales[[another.scaleind]]$secondary.axis, "waiver")){
            return("another.secondary.axis")
        }else{
            return('normal.position')
        }
    }else{
        return('normal.position')
    }
}

#' @importFrom ggplot2 coord_flip
check_coord_flip <- function(plot){
    if (inherits(plot, "gg") && inherits(plot$coordinates, "CoordFlip")){
        return("coord_flip")
    }
    return ("coord_cartesian")
}

compute_relative_range <- function(breaks, scales, rng){
    if(rng$flagrev=="reverse"){
        baserange <- abs(diff(rev(breaks)[[1]]))
        otherbk <- breaks[-length(breaks)]
    }else{ 
        baserange <- abs(diff(breaks[[1]]))
        otherbk <- breaks[-1]
    }
    relranges <- unlist(mapply(compute_relative_range_, 
                     breaks_= otherbk, 
                     scales_=scales, 
                     MoreArgs=list(baserange_=baserange), 
                     #baserange_ = baserange,
                     SIMPLIFY=FALSE))
    if (rng$flagrev == "reverse"){
        return (c(relranges, baserange)) 
    }else{
        return (c(baserange, relranges))
    }
}

compute_relative_range_ <- function(breaks_, scales_, baserange_){
    if (scales_=="fixed"){
        return(abs(diff(breaks_)))
    }
    if (scales_=="free"){
        scales_ = 1
    }
    if (!is_numeric(scales_) || length(scales_) > 1){
        abort("The scales must be a numeric or one of 'fixed', 'free' !")
    }
    relrange <- baserange_ * as.numeric(scales_)
    return (relrange)
}

is_numeric <- function(x) {
    !anyNA(suppressWarnings(as.numeric(x)))
}

check_xy_intercept <- function(plot){
    confuse_xy_labs <- c("xend", "xmax", "xmin", "xintercept", 
                         "yend", "ymax", "ymin", "yintercept")
    index <- confuse_xy_labs[confuse_xy_labs %in% names(ggplot_build(plot)$plot$labels)]
    if (length(index) == 0){
       return(plot)
    }
    params <- lapply(seq(length(index)), function(i)NULL)
    names(params) <- index
    plot <- plot + do.call('labs', params)
    return (plot)
}

add_expand <- function(plot, expand, axis){
   expand <- convert_expand(expand=expand)
   var <- paste0("panel_scales_", axis)
   gb <- ggplot_build(plot)
   scales_axis_obj <- gb$layout[[var]][[1]]
   scales_axis_obj$expand <- expand
   plot <- suppressWarnings(plot + scales_axis_obj)
   return(plot)
}

convert_expand <- function(expand){
   if (!is.numeric(expand) && !expand){
       expand <- c(0, 0)
   }
   if (!is.numeric(expand) && expand){
       expand <- ggplot2::waiver()
   }
   return(expand)
}

## Theme for 2D grid cells (dual-axis breaks)
## col_type: "first" (left), "other" (middle), "last" (right) — controls y-axis visibility
## row_type: "first" (bottom), "other" (middle), "last" (top) — controls x-axis visibility
subplot_theme_2d <- function(plot, col_type, row_type, margin_x, margin_y, rev_x, rev_y, symbol_x = NULL, symbol_y = NULL) {
    te <- theme()

    # Y-axis visibility: hide for non-first columns
    if (col_type %in% c("other", "last")) {
        te <- te + axis_theme(plot, axis = "x")
    }

    # X-axis visibility: hide for non-bottom rows
    if (row_type %in% c("other", "last")) {
        te <- te + axis_theme(plot, axis = "y")
    }

    # Y-strips: show only in last column (rightmost)
    if (col_type != "last") {
        te <- te + strip_theme(plot, axis = "x")
    }

    # X-strips: show only in last row (topmost = "last" type)
    if (row_type != "last") {
        te <- te + strip_theme(plot, axis = "y")
    }

    # Combined margins from both axes
    l <- if (col_type == "first") 0 else margin_x / 2
    r <- if (col_type == "last")  0 else margin_x / 2
    t <- if (row_type == "last")  0 else margin_y / 2
    b <- if (row_type == "first") 0 else margin_y / 2

    if (rev_x == "reverse") { tmp <- l; l <- r; r <- tmp }
    if (rev_y == "reverse") { tmp <- t; t <- b; b <- tmp }

    te <- te + theme(plot.margin = margin(t = t, r = r, b = b, l = l, unit = "cm"))

    # Handle panel border for boxed themes (theme_bw, theme_linedraw, etc.)
    # Remove the full rectangle and draw lines only on outer-facing edges
    border_te <- .border_theme_2d(plot, col_type, row_type)
    if (!is.null(border_te)) {
        te <- te + border_te
    }

    # Add break symbols if requested
    if (!is.null(symbol_x) || !is.null(symbol_y)) {
        sym_te <- .symbol_theme_2d(te, plot, col_type, row_type, symbol_x, symbol_y, rev_x, rev_y)
        if (!is.null(sym_te)) {
            te <- te + sym_te
        }
    }

    return(te)
}

.symbol_theme_2d <- function(te, plot, col_type, row_type, symbol_x, symbol_y, rev_x, rev_y) {
    # Determine the current border from the composed theme or plot theme
    full_theme <- ggplot2::theme_get() + plot$theme + te
    border <- full_theme$panel.border

    sides_x <- character(0)
    if (!is.null(symbol_x)) {
        # Only add X symbols on the top/bottom outer edges, so we check row_type
        # For X-break (vertical gap), we want symbols on the top/bottom edges of the gap.
        # But in a 2D grid, the gap exists between columns.
        # So we want symbols on the top/bottom edges of the *plot*, at the column boundaries.
        # The column boundaries are between col_type="first" and "other", etc.
        # Wait, if we attach the symbol to the panel border, it will draw on the panel's edge.
        # If we attach to col="first", side="right", it draws on the right edge of the first column.
        # We only want to draw this IF this panel is at the top row or bottom row.
        
        if (row_type == "first" || row_type == "last") {
            # Only draw X-break symbols if we are in the top or bottom row
            if (col_type == "first") {
                sides_x <- if (rev_x == "reverse") "left" else "right"
            } else if (col_type == "last") {
                sides_x <- if (rev_x == "reverse") "right" else "left"
            } else if (col_type == "other") {
                sides_x <- c("left", "right")
            }
        }
    }

    sides_y <- character(0)
    if (!is.null(symbol_y)) {
        # Only add Y symbols on the left/right outer edges, so we check col_type
        if (col_type == "first" || col_type == "last") {
            # Only draw Y-break symbols if we are in the left or right column
            if (row_type == "first") {
                sides_y <- if (rev_y == "reverse") "bottom" else "top"
            } else if (row_type == "last") {
                sides_y <- if (rev_y == "reverse") "top" else "bottom"
            } else if (row_type == "other") {
                sides_y <- c("top", "bottom")
            }
        }
    }
    
    # However, `element_break_symbol` logic for "dual" axis currently draws on BOTH top and bottom corners
    # if `axis="dual"` or "x".
    # If we are in the top row (row_type="last"), we only want the TOP symbol if there is a border,
    # and maybe we don't want the bottom symbol because the bottom symbol is "inside" the plot?
    # No, for X-break, the gap is vertical. The symbol should appear on the X-axis line.
    # In a 2D grid, the X-axis is usually at the bottom of the bottom row.
    # So for `row_type="first"` (bottom row), we want the symbol on the BOTTOM edge.
    # For `row_type="last"` (top row), we generally don't have an X-axis unless it's `scale_x_top`.
    # But if there is a full border (boxed), we might want it on the top edge of the top row too.
    # Crucially, we do NOT want symbols on the top edge of the bottom row, or bottom edge of the top row (internal edges).
    
    # We need to pass more info to `element_break_symbol` or refine `sides`.
    # Let's refine `sides` to be very specific: "bottom-right", "top-right", etc?
    # `element_break_symbol` uses "right", "left", "top", "bottom" to choose which GROBS to generate.
    # If we pass "right", it generates both top-right and bottom-right slashes (if boxed) or just bottom-right (if not boxed).
    # But here we need finer control.
    # If row_type="first" (bottom), and we have an X-break on the right edge, we ONLY want the BOTTOM-right slash.
    # If row_type="last" (top), and we have an X-break on the right edge, we ONLY want the TOP-right slash (and only if boxed).
    
    # The current `element_break_symbol` implementation for `axis="dual"` or "x":
    # if "right" in sides: draws bottom-right. If has_border, also draws top-right.
    # This "automagic" behavior is causing the issue. It assumes if you are on the right edge, you want both corners if boxed.
    # But in the middle of the plot (internal edges), we might have a border but we don't want the internal corner symbol.
    
    # To fix this without changing `element_break_symbol` structure too much:
    # We can pass a new argument `corner_mask` or `draw_top`/`draw_bottom`.
    # Or, simpler:
    # We can define `sides` more granularly in `element_break_symbol`? No, it expects standard side names.
    
    # Let's modify `element_break_symbol` to accept `clip_corners` or similar?
    # Or just let `.symbol_theme_2d` decide exactly which corners.
    
    # Let's change the logic in `element_break_symbol` to strictly follow what we need.
    # Actually, we can use `axis="x_bottom"` or `axis="x_top"`?
    
    # Let's update `element_break_symbol` to handle `which` argument?
    # Or we can just be smart in `.symbol_theme_2d` and passing a custom `axis` string is a hacky but effective way.
    # e.g. axis="x_bottom" -> only draw bottom corners.
    
    # Let's refine `.symbol_theme_2d` logic:
    
    sides <- c()
    specific_axis <- "dual" # default
    
    # We need to separate X and Y symbol addition or combine them carefully.
    # Actually, we can return a list of theme elements or combine them.
    # But `panel.border` is unique.
    
    # Strategy:
    # 1. Calculate which specific corners need symbols.
    # 2. Pass this specific instruction to `element_break_symbol`.
    
    # Let's allow `element_break_symbol` to take a `corners` argument? 
    # e.g. corners = c("tr", "br", "tl", "bl")
    
    # Let's update `element_break_symbol` first (I will do this in the next step).
    # Here I will prepare the logic to determine corners.
    
    corners <- character(0)
    
    # Logic for X-breaks (vertical gaps between cols)
    if (!is.null(symbol_x)) {
        # We only care if we are in the bottom row (standard x-axis) or top row (if boxed).
        # We NEVER want symbols on the internal horizontal edges (top of bottom row, bottom of top row).
        
        is_bottom_row <- (row_type == "first")
        is_top_row <- (row_type == "last")
        
        if (is_bottom_row || is_top_row) {
            # Determine which vertical edge has the break
            edge <- NULL
            if (col_type == "first") edge <- if (rev_x == "reverse") "left" else "right"
            else if (col_type == "last") edge <- if (rev_x == "reverse") "right" else "left"
            else if (col_type == "other") edge <- c("left", "right")
            
            if (!is.null(edge)) {
                if ("right" %in% edge) {
                    if (is_bottom_row) corners <- c(corners, "x_bottom_right")
                    if (is_top_row) corners <- c(corners, "x_top_right")
                }
                if ("left" %in% edge) {
                    if (is_bottom_row) corners <- c(corners, "x_bottom_left")
                    if (is_top_row) corners <- c(corners, "x_top_left")
                }
            }
        }
    }
    
    # Logic for Y-breaks (horizontal gaps between rows)
    if (!is.null(symbol_y)) {
        # We only care if we are in the left col (standard y-axis) or right col (if boxed).
        
        is_left_col <- (col_type == "first")
        is_right_col <- (col_type == "last")
        
        if (is_left_col || is_right_col) {
            # Determine which horizontal edge has the break
            edge <- NULL
            if (row_type == "first") edge <- if (rev_y == "reverse") "bottom" else "top"
            else if (row_type == "last") edge <- if (rev_y == "reverse") "top" else "bottom"
            else if (row_type == "other") edge <- c("top", "bottom")
            
            if (!is.null(edge)) {
                if ("top" %in% edge) {
                    if (is_left_col) corners <- c(corners, "y_top_left")
                    if (is_right_col) corners <- c(corners, "y_top_right")
                }
                if ("bottom" %in% edge) {
                    if (is_left_col) corners <- c(corners, "y_bottom_left")
                    if (is_right_col) corners <- c(corners, "y_bottom_right")
                }
            }
        }
    }
    
    if (length(corners) > 0) {
        # Pass the calculated corners to the element
        symbol <- if (!is.null(symbol_x)) symbol_x else symbol_y
        return(theme(panel.border = element_break_symbol(symbol = symbol, corners = corners, base_border = border, axis = "dual_custom")))
    }
    return(NULL)
}

## Custom theme element: break symbol
element_break_symbol <- function(symbol = "slash", colour = "black", linewidth = 0.5,
                                 linetype = 1, sides = "right", base_border = NULL, axis = "x", corners = NULL) {
    structure(
        list(symbol = symbol, colour = colour, linewidth = linewidth, linetype = linetype,
             sides = sides, base_border = base_border, axis = axis, corners = corners, fill = NA, inherit.blank = FALSE),
        class = c("element_break_symbol", "element_rect", "element")
    )
}

#' @method element_grob element_break_symbol
#' @export
element_grob.element_break_symbol <- function(element, ...) {
    grobs <- list()
    has_border <- !is.null(element$base_border) && !inherits(element$base_border, "element_blank")
    if (has_border) {
        grobs <- c(grobs, list(ggplot2::element_grob(element$base_border, ...)))
    }
    
    gp <- grid::gpar(
        col = element$colour,
        lwd = element$linewidth * 72.27 / 25.4,   # mm -> points
        lty = element$linetype
    )
    
    sides <- element$sides
    symbol <- element$symbol
    axis <- element$axis
    corners <- element$corners
    
    if (symbol == "slash") {
        # Modified slash style: slant is adjusted to 0.5mm to be nearly vertical/horizontal
        slant <- grid::unit(0.6, "mm") 
        height <- grid::unit(3.5, "mm") # y displacement
        shift <- grid::unit(1.0, "mm")  # separation

        # Helper function to add a pair of slashes at a given corner
        add_slash <- function(x_npc, y_npc, orient = "vertical") {
            if (orient == "vertical") {
                # For X-break gap (vertical gap), the slash crosses horizontally
                list(
                    grid::segmentsGrob(
                        x0 = grid::unit(x_npc, "npc") - shift - slant, y0 = grid::unit(y_npc, "npc") - height,
                        x1 = grid::unit(x_npc, "npc") - shift + slant, y1 = grid::unit(y_npc, "npc") + height,
                        gp = gp),
                    grid::segmentsGrob(
                        x0 = grid::unit(x_npc, "npc") + shift - slant, y0 = grid::unit(y_npc, "npc") - height,
                        x1 = grid::unit(x_npc, "npc") + shift + slant, y1 = grid::unit(y_npc, "npc") + height,
                        gp = gp)
                )
            } else {
                # For Y-break gap (horizontal gap), the slash crosses vertically
                list(
                    grid::segmentsGrob(
                        x0 = grid::unit(x_npc, "npc") - height, y0 = grid::unit(y_npc, "npc") - shift - slant,
                        x1 = grid::unit(x_npc, "npc") + height, y1 = grid::unit(y_npc, "npc") - shift + slant,
                        gp = gp),
                    grid::segmentsGrob(
                        x0 = grid::unit(x_npc, "npc") - height, y0 = grid::unit(y_npc, "npc") + shift - slant,
                        x1 = grid::unit(x_npc, "npc") + height, y1 = grid::unit(y_npc, "npc") + shift + slant,
                        gp = gp)
                )
            }
        }

        # If "corners" are provided (Dual Axis 2D mode with precise control)
        if (!is.null(corners) && axis == "dual_custom") {
            # X-axis symbols (Vertical cut)
            if ("x_bottom_right" %in% corners) grobs <- c(grobs, add_slash(1, 0, "vertical"))
            if ("x_top_right" %in% corners) grobs <- c(grobs, add_slash(1, 1, "vertical"))
            if ("x_bottom_left" %in% corners) grobs <- c(grobs, add_slash(0, 0, "vertical"))
            if ("x_top_left" %in% corners) grobs <- c(grobs, add_slash(0, 1, "vertical"))
            
            # Y-axis symbols (Horizontal cut)
            if ("y_top_left" %in% corners) grobs <- c(grobs, add_slash(0, 1, "horizontal"))
            if ("y_top_right" %in% corners) grobs <- c(grobs, add_slash(1, 1, "horizontal"))
            if ("y_bottom_left" %in% corners) grobs <- c(grobs, add_slash(0, 0, "horizontal"))
            if ("y_bottom_right" %in% corners) grobs <- c(grobs, add_slash(1, 0, "horizontal"))
        } else {
            # Standard 1D behavior (fallback)
            if (axis %in% c("x", "dual")) {
                if ("right" %in% sides) {
                    grobs <- c(grobs, add_slash(1, 0, "vertical"))
                    if (has_border) grobs <- c(grobs, add_slash(1, 1, "vertical"))
                }
                if ("left" %in% sides) {
                    grobs <- c(grobs, add_slash(0, 0, "vertical"))
                    if (has_border) grobs <- c(grobs, add_slash(0, 1, "vertical"))
                }
            }
            
            if (axis %in% c("y", "dual")) {
                if ("top" %in% sides) {
                    grobs <- c(grobs, add_slash(0, 1, "horizontal"))
                    if (has_border) grobs <- c(grobs, add_slash(1, 1, "horizontal"))
                }
                if ("bottom" %in% sides) {
                    grobs <- c(grobs, add_slash(0, 0, "horizontal"))
                    if (has_border) grobs <- c(grobs, add_slash(1, 0, "horizontal"))
                }
            }
        }
    } else if (symbol == "zigzag") {
        # Optional
    }

    if (length(grobs) == 0) return(grid::nullGrob())
    do.call(grid::grobTree, grobs)
}
## Used for panel.border in 2D break grids to avoid doubled borders.
## Useful for for theme where there is a panel border (e.g. theme_bw, linedraw) and we want to avoid drawing borders on internal edges of the 2D grid.
element_partial_rect <- function(colour = "grey20", linewidth = 0.5,
                                 linetype = 1, sides = c("top", "bottom", "left", "right")) {
    structure(
        list(colour = colour, linewidth = linewidth, linetype = linetype,
             sides = sides, fill = NA, inherit.blank = FALSE),
        class = c("element_partial_rect", "element_rect", "element")
    )
}

#' @importFrom ggplot2 element_grob
#' @method element_grob element_partial_rect
#' @export
element_grob.element_partial_rect <- function(element, ...) {
    gp <- grid::gpar(
        col = element$colour,
        lwd = element$linewidth * 72.27 / 25.4,   # mm -> points
        lty = element$linetype
    )

    grobs <- list()
    sides <- element$sides
    if ("left" %in% sides)
        grobs <- c(grobs, list(grid::segmentsGrob(
            x0 = 0, y0 = 0, x1 = 0, y1 = 1, default.units = "npc", gp = gp)))
    if ("right" %in% sides)
        grobs <- c(grobs, list(grid::segmentsGrob(
            x0 = 1, y0 = 0, x1 = 1, y1 = 1, default.units = "npc", gp = gp)))
    if ("bottom" %in% sides)
        grobs <- c(grobs, list(grid::segmentsGrob(
            x0 = 0, y0 = 0, x1 = 1, y1 = 0, default.units = "npc", gp = gp)))
    if ("top" %in% sides)
        grobs <- c(grobs, list(grid::segmentsGrob(
            x0 = 0, y0 = 1, x1 = 1, y1 = 1, default.units = "npc", gp = gp)))

    if (length(grobs) == 0) return(grid::nullGrob())
    do.call(grid::grobTree, grobs)
}

## Replace panel.border rectangle with a partial-rect that draws only
## the outer-facing edges of each cell in the 2D grid.
.border_theme_2d <- function(plot, col_type, row_type) {
    # Resolve panel.border from plot theme + global default
    full_theme <- ggplot2::theme_get() + plot$theme
    border <- full_theme$panel.border
    if (is.null(border) || inherits(border, "element_blank")) {
        return(NULL)
    }

    # Extract border appearance
    border_colour <- if (!is.null(border$colour)) border$colour else "grey20"
    border_lw <- if (!is.null(border$linewidth)) border$linewidth
                 else if (!is.null(border$size)) border$size
                 else 0.5

    # Draw only outer-facing edges of the grid perimeter
    sides <- character(0)
    if (col_type == "first") sides <- c(sides, "left")
    if (col_type == "last")  sides <- c(sides, "right")
    if (row_type == "first") sides <- c(sides, "bottom")
    if (row_type == "last")  sides <- c(sides, "top")

    theme(
        panel.border = element_partial_rect(
            colour = border_colour, linewidth = border_lw, sides = sides
        )
    )
}

---
title: "Set Axis Break for ggplot2"
author: "Guangchuang Yu and Shuangbin Xu\\

        School of Basic Medical Sciences, Southern Medical University"
date: "`r Sys.Date()`"
output:
  prettydoc::html_pretty:
    toc: true
    theme: cayman
    highlight: github
  pdf_document:
    toc: true
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{ggbreak introduction}
  %\VignetteDepends{ggplot2}
  %\VignetteDepends{grid}
  %\VignetteDepends{aplot}
  %\VignetteDepends{ggplotify}
  %\usepackage[utf8]{inputenc}
---



```{r include=FALSE}
knitr::opts_chunk$set(warning = FALSE,
                      message = TRUE)

library(ggplot2)
library(ggbreak)
library(patchwork)
library(yulab.utils)
```

## Introduction

This package was first designed to set breakpoints for truncating the plot as I need to [shrink outlier long branch of a phylogenetic tree](https://yulab-smu.top/treedata-book/faq.html#shrink-outlier-long-branch).


Axis break or a so-called gap plot is useful for large datasets that are not normally distributed and contain outliers. Sometimes we can transform the data (e.g. using log-transformation if the data was log-normal distributed) to solve this problem. But this is not always granted. The data may just simply contain outliers and these outliers are meaningful. A simple gap plot can solve this issue well to present the data in detail with both normal and extreme data. 


This package provides several scale functions to break down a 'gg' plot into pieces and align them together with (gap plot) or without (wrap plot or cut plot) ignoring subplots. Our methods are fully compatible with `ggplot2`, so that users can still use the `+` operator to add geometric layers after creating a broken axis.



If you use `r CRANpkg('ggbreak')` in published research, please cite the following paper:

+ S Xu<sup>#</sup>, M Chen<sup>#</sup>, T Feng, L Zhan, L Zhou, __G Yu__<sup>\*</sup>. Use ggbreak to effectively utilize plotting space to deal with large datasets and outliers. __*Frontiers in Genetics*__. 2021, 12:774846. doi: [10.3389/fgene.2021.774846](https://www.frontiersin.org/articles/10.3389/fgene.2021.774846/full/)


## Gap plot

For creating gap plot, we provide `scale_x_break` and `scale_y_break` functions. Multiple breakpoints on a single axis are supported, and you can also apply both functions to set breakpoints for both x and y axes simultaneously.

### Feature 1: Compatible with ggplot2

After breaking the plot, we can still superpose geometric layers and set themes. This ensures that users familiar with `ggplot2` can seamlessly adopt `ggbreak` without changing their workflow. The following example demonstrates adding a text layer and modifying the theme after applying an axis break.

```{r fig.keep="last"}
library(ggplot2)
library(ggbreak) 
library(patchwork)

set.seed(2019-01-19)
d <- data.frame(x = 1:20,
   y = c(rnorm(5) + 4, rnorm(5) + 20, rnorm(5) + 5, rnorm(5) + 22)
)
 
p1 <- ggplot(d, aes(y, x)) + geom_col(orientation="y")
d2 <- data.frame(x = c(2, 18), y = c(7, 26), label = c("hello", "world"))
p2 <- p1 + scale_x_break(c(7, 17)) + 
  geom_text(aes(y, x, label=label), data=d2, hjust=1, colour = 'firebrick')  + 
  xlab(NULL) + ylab(NULL) + theme_minimal()

p1 + p2
```

### Feature 2: Multiple break-points are supported

`ggbreak` allows users to specify multiple breakpoints on a single axis. This is particularly useful when the data contains multiple clusters of outliers or interesting regions separated by large empty intervals. You can simply add multiple `scale_x_break()` layers to the plot.

```{r}
p2 + scale_x_break(c(18, 21))
```

### Feature 3: Simultaneous breaks on both x and y axes

You can combine `scale_x_break()` and `scale_y_break()` to create dual-axis break plots. This is particularly useful when you have data points that form distinct clusters with large gaps in between on both dimensions. The package handles the layout and alignment automatically.

```{r fig.width=8, fig.height=6}
set.seed(2023-01-01)
df <- data.frame(
    x = c(rnorm(50, 5, 1), rnorm(10, 50, 2)),
    y = c(rnorm(50, 10, 2), rnorm(10, 100, 5)),
    group = c(rep("Cluster 1", 50), rep("Cluster 2", 10))
)

ggplot(df, aes(x, y, color = group)) + 
    geom_point(size = 3) +
    scale_x_break(c(10, 45)) + 
    scale_y_break(c(20, 90)) +
    theme_bw() +
    theme(legend.position = "top")
```

Multiple breaks on both axes are also supported. This feature enables complex visualizations where data is distributed across multiple disjoint regions in a 2D space.

```{r fig.width=8, fig.height=6}
ggplot(df, aes(x, y, color = group)) + 
    geom_point(size = 3) +
    scale_x_break(c(10, 20)) + 
    scale_x_break(c(30, 45)) + 
    scale_y_break(c(20, 40)) + 
    scale_y_break(c(60, 90)) +
    theme_bw() +
    theme(legend.position = "top")
```

### Feature 4: Axis break symbols

You can add standard axis break symbols (like a double-slash `//`) to the axes where they are broken by using the `symbol` parameter. This visual cue helps readers quickly identify that the axis is discontinuous. Currently, `symbol = "slash"` is supported, and it works for both single and dual axis breaks.

```{r fig.width=8, fig.height=4}
ggplot(mpg, aes(displ, hwy)) +
    geom_point() +
    scale_x_break(c(3, 4), symbol = "slash")
```

### Feature 5: Zoom in or zoom out of subplots

The `scales` parameter allows you to control the relative size of the subplots. This is useful when you want to zoom in on a specific range of data to show more detail, or zoom out to show the overall trend. A value larger than 1 zooms in (allocates more space), while a value smaller than 1 zooms out.

```{r}
p1 + scale_x_break(c(7, 17), scales = 1.5) + scale_x_break(c(18, 21), scales=2)
```

### Feature 6: Support reverse scale

`ggbreak` works seamlessly with `scale_y_reverse()` (and `scale_x_reverse()`). This is common in fields like oceanography or atmospheric science where depth or pressure is plotted on a reversed axis. The break function respects the reversed direction of the axis.

```{r fig.keep='last'}
g <- ggplot(d, aes(x, y)) + geom_col()
g2 <- g + scale_y_break(c(7, 17), scales = 1.5) + 
  scale_y_break(c(18, 21), scale=2) + scale_y_reverse()
g + g2
```

### Feature 7: Compatible with scale transform functions

Users can apply scale transform functions, such as `scale_x_log10` and `scale_x_sqrt`, to an axis break plot. This allows for handling data that spans several orders of magnitude while still excluding uninteresting ranges.

```{r fig.keep='last', fig.width=10, fig.height=5}
p2 <- p1 + scale_x_break(c(7, 17)) 
p3 <- p1 + scale_x_break(c(7, 17)) + scale_x_log10()
p2 + p3
```

### Feature 8: Compatible with `coord_flip`

Flipping the coordinate system with `coord_flip()` is fully supported. This is often used to create horizontal bar charts or to swap axes for better readability. `ggbreak` detects the flip and adjusts the axis breaks accordingly.

```{r message=FALSE}
g + coord_flip() + scale_y_break(c(7, 18))
```


### Feature 9: Compatible with `facet_grid` and `facet_wrap`

`ggbreak` can be used in conjunction with faceting functions like `facet_grid()` and `facet_wrap()`. This allows you to create small multiples where each panel has a broken axis, which is extremely powerful for comparing distributions across different groups.

```{r message=FALSE}
set.seed(2019-01-19)
d <- data.frame(
  x = 1:20,
  y = c(rnorm(5) + 4, rnorm(5) + 20, rnorm(5) + 5, rnorm(5) + 22),
  group = c(rep("A", 10), rep("B", 10)),
  face=c(rep("C", 5), rep("D", 5), rep("E", 5), rep("F", 5))
)

p <- ggplot(d, aes(x=x, y=y)) +
     geom_col(orientation="x") +
     scale_y_reverse() +
     facet_wrap(group~.,
                scales="free_y",
                strip.position="right",
                nrow=2
                ) +
     coord_flip()
pg <- p +
  scale_y_break(c(7, 17), scales="free") +
  scale_y_break(c(19, 21), scales="free")
print(pg)
```



### Feature 10: Compatible with legends

Legends are automatically handled and preserved. You can position the legend anywhere using `theme(legend.position = ...)`. In this example, we move the legend to the bottom of the plot.

```{r message=FALSE}
pg <- pg + aes(fill=group) + theme(legend.position = "bottom")
print(pg)
```

### Feature 11: Supports all plot labels

All standard plot labels, including title, subtitle, caption, and tag, are supported and correctly placed around the broken plot. Standard theme elements for these labels (like font size, face, and position) are also respected.

```{r message=FALSE}
pg + labs(title="test title", subtitle="test subtitle", tag="A tag", caption="A caption") +
     theme_bw() +
     theme(
           legend.position = "bottom",
           strip.placement = "outside",
           axis.title.x=element_text(size=10),
           plot.title = element_text(size = 22),
           plot.subtitle = element_text(size = 16),
           plot.tag = element_text(size = 10),
           plot.title.position = "plot",
           plot.tag.position = "topright",
           plot.caption = element_text(face="bold.italic"),

     )
```

### Feature 12: Allows setting tick labels for subplots

Sometimes you might want specific control over the tick labels in each subplot segment. The `ticklabels` argument allows you to manually specify which labels should appear in each broken segment, overriding the default breaks.

```{r message=FALSE, fig.width=10, fig.height=6}
require(ggplot2)
library(ggbreak)
set.seed(2019-01-19)
d <- data.frame(
  x = 1:20,
  y =  c(rnorm(5) + 4, rnorm(5) + 20, rnorm(5) + 5, rnorm(5) + 22),
  group = c(rep("A", 10), rep("B", 10))
)

p <- ggplot(d, aes(x=x, y=y)) +
     scale_y_reverse() +
     scale_x_reverse() +
     geom_col(aes(fill=group)) +
     scale_fill_manual(values=c("#00AED7", "#009E73")) +
     facet_wrap(
         group~.,
         scales="free_y",
         strip.position="right",
         nrow=2
     ) +
     coord_flip()                                                                                                                                                                                                  

p +
     scale_y_break(c(7, 10), scales=0.5, ticklabels=c(10, 11.5, 13)) +
     scale_y_break(c(13, 17), scales=0.5, ticklabels=c(17, 18, 19)) +
     scale_y_break(c(19,21), scales=1, ticklabels=c(21, 22, 23))
```

### Feature 13: Compatible with dual axis

`ggbreak` works correctly with `scale_y_continuous(sec.axis = ...)` to create dual y-axes (e.g., metric vs imperial units). The secondary axis is broken in sync with the primary axis.

```{r fig.width=10, fig.height=5}
p <- ggplot(mpg, aes(displ, hwy)) +
     geom_point() +
     scale_y_continuous(
       "mpg (US)",
       sec.axis = sec_axis(~ . * 1.20, name = "mpg (UK)")
     ) +
      theme(
        axis.title.y.left = element_text(color="deepskyblue"),
        axis.title.y.right = element_text(color = "orange")
      )
p1 <- p + scale_y_break(breaks = c(20, 30))
p2 <- p + scale_x_break(breaks = c(3, 4))
p1 + p2
```

### Feature 14: Compatible with patchwork

`ggbreak` objects are fully compatible with `patchwork`. This means you can combine multiple broken plots, or combine broken plots with standard ggplot objects, into a single composite figure using simple arithmetic operators like `+` or `/`.

```{r message=FALSE, fig.width=8, fig.height=5, fig.keep="last"}
library(patchwork)

set.seed(2019-01-19)
d <- data.frame(
               x = 1:20,
               y = c(rnorm(5) + 4, rnorm(5) + 20, rnorm(5) + 5, rnorm(5) + 22)
)

p <- ggplot(d, aes(x, y)) + geom_col()
x <- p+scale_y_break(c(7, 17 ))

x + p
```

## Wrap plot

The `scale_wrap()` function wraps a 'gg' plot over multiple rows to make plots with long x-axes easier to read.

```{r fig.width=6, fig.height=8}
p <- ggplot(economics, aes(x=date, y = unemploy, colour = uempmed)) +
  geom_line()

p + scale_wrap(n=4)
```


Both categorical and numerical variables are supported.

```{r fig.width=7, fig.height=6}
ggplot(mpg, aes(class, hwy)) + 
  geom_boxplot() +
      scale_wrap(n = 2)
```

## Cut plot

The `scale_x_cut` or `scale_y_cut` cuts a 'gg' plot to several slices with the ability to specify which subplots to zoom in or zoom out.

```{r}
library(ggplot2)
library(ggbreak)
set.seed(2019-01-19)
d <- data.frame(
     x = 1:20,
     y = c(rnorm(5) + 4, rnorm(5) + 20, rnorm(5) + 5, rnorm(5) + 22)
 )
p <- ggplot(d, aes(x, y)) + geom_col()
p + scale_y_cut(breaks=c(7, 18), which=c(1, 3), scales=c(3, 0.5))
```

## Adjust the amount of space between subplots

The `space` parameter in `scale_x_break()`, `scale_y_break()`, `scale_x_cut()` and `scale_y_cut()` allows user to control the space between subplots.


```{r}
p + scale_y_cut(breaks=c(7, 18), which=c(1, 3), scales=c(3, 0.5), space=.5)
```

## Place legend at any position


```{r legend-subview, fig.keep="last"}
## original plot
p1 <- ggplot(mpg, aes(displ, hwy, color=factor(cyl))) + geom_point()

## ggbreak plot without legend
p2 <- p1 + scale_x_break(c(3, 4)) +
    theme(legend.position="none") 

## extract legend from original plot
leg = cowplot::get_legend(p1)

## redraw the figure
p3 <- ggplotify::as.ggplot(print(p2))

## place the legend 
p3 + ggimage::geom_subview(x=.9, y=.8, subview=leg)
```



## Note

The features we introduced for `scale_x_break` and `scale_y_break` also work for `scale_wrap`, `scale_x_cut` and `scale_y_cut`.

## FAQ

1. Incompatible with functions that arrange multiple plots

You can use `aplot::plot_list()` to arrange `ggbreak` objects with other `ggplot` objects. For other functions, such as `cowplot::plot_grid()` and `gridExtra::grid.arrange()`, you need to explictly call `print()` to `ggbreak` object, see also <https://github.com/YuLab-SMU/ggbreak/issues/36>.

Using `print()` is a secret magic to make `ggbreak` compatible with other packages, including [export](https://github.com/YuLab-SMU/ggbreak/issues/37). 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{scale_x_cut}
\alias{scale_x_cut}
\alias{scale_y_cut}
\title{scale_x_cut}
\usage{
scale_x_cut(breaks, which = NULL, scales = NULL, expand = FALSE, space = 0.1)

scale_y_cut(breaks, which = NULL, scales = NULL, expand = FALSE, space = 0.1)
}
\arguments{
\item{breaks}{a numeric or numeric vector, the points to be divided}

\item{which}{integer, the position of subplots to scales, started from left to right or top to bottom.}

\item{scales}{numeric, relative width or height of subplots.}

\item{expand}{default is FALSE, logical a vector of range expansion constants
used to add some padding around the data to ensure that they
are placed some distance away from the axes. Use the convenience
function \code{expansion()} of \code{ggplot2} to generate the values for
the \code{expand} argument. The defaults are to expand the scale
by 5\% on each side for continuous variables. If it is logical, the \code{TRUE} means
the default of \code{ggplot2} (foregoing statement), and \code{FALSE} means no expand for the plot.}

\item{space}{the blank space among the subplots after cut, default is 0.1 (cm).}
}
\value{
gg object
}
\description{
scale_x_cut

scale_y_cut
}
\examples{
library(ggplot2)
library(ggbreak)
set.seed(2019-01-19)
d <- data.frame(
     x = 1:20,
     y = c(rnorm(5) + 4, rnorm(5) + 20, rnorm(5) + 5, rnorm(5) + 22)
 )
p <- ggplot(d, aes(x, y)) + geom_col()
p + scale_y_cut(breaks=c(7, 18), which=c(1, 3), scales=c(3, 0.5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggbreak-package.R
\docType{package}
\name{ggbreak-package}
\alias{ggbreak}
\alias{ggbreak-package}
\title{ggbreak: Set Axis Break for 'ggplot2'}
\description{
An implementation of scale functions for setting axis breaks of a 'gg' plot (S Xu (2021) \doi{10.3389/fgene.2021.774846}).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/YuLab-SMU/ggbreak}
  \item Report bugs at \url{https://github.com/YuLab-SMU/ggbreak/issues}
}

}
\author{
\strong{Maintainer}: Guangchuang Yu \email{guangchuangyu@gmail.com} (\href{https://orcid.org/0000-0002-6485-8781}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Shuangbin Xu \email{xshuangbin@163.com} (\href{https://orcid.org/0000-0003-3513-5362}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{scale_x_break}
\alias{scale_x_break}
\alias{scale_y_break}
\title{scale_x_break}
\usage{
scale_x_break(
  breaks,
  scales = "fixed",
  ticklabels = NULL,
  expand = TRUE,
  space = 0.1,
  symbol = NULL
)

scale_y_break(
  breaks,
  scales = "fixed",
  ticklabels = NULL,
  expand = TRUE,
  space = 0.1,
  symbol = NULL
)
}
\arguments{
\item{breaks}{break point}

\item{scales}{relative width or height of subplots,
default is "fixed". If scale is 'free', all subplots have
equal width or height. It also can be any number to set
relative width or height compare to first subplot.}

\item{ticklabels}{the axis labels to subplot, default is NULL.}

\item{expand}{default is TRUE, logical or a vector of range expansion constants
used to add some padding around the data to ensure that they
are placed some distance away from the axes. Use the convenience
function \code{expansion()} of \code{ggplot2} to generate the values for
the \code{expand} argument. The defaults are to expand the scale
by 5\% on each side for continuous variables. If it is logical, the \code{TRUE} means
the default of \code{ggplot2} (foregoing statement), and \code{FALSE} means no expand for the plot.}

\item{space}{the blank space among the subplots after break, default is 0.1 (cm).}

\item{symbol}{the break symbol, default is NULL, could be 'slash'.}
}
\value{
gg object
}
\description{
Set an axis break point for a 'gg' plot
}
\details{
This scale function set an axis break point for a 'gg' plot. Either 'x' and 'y' axes are supported. The result is still a 'gg' object and user can progressively add layers to it.
}
\examples{
require(ggplot2 )
set.seed(2019-01-19)
d <- data.frame(
  x = 1:20,
  y = c(rnorm(5) + 4, rnorm(5) + 20, rnorm(5) + 5, rnorm(5) + 22)
)

p <- ggplot(d, aes(x, y)) + geom_col()
x <- p+scale_y_break(c(7, 17 ) )
print(x)
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scales.R
\name{scale_wrap}
\alias{scale_wrap}
\title{scale-wrap}
\usage{
scale_wrap(n)
}
\arguments{
\item{n}{the number of subplot pieces.}
}
\value{
gg object
}
\description{
This scale function wraps a 'gg' plot over multiple rows to make plots with long x axes easier to read.
}
\examples{
library(ggplot2)
library(ggbreak)
p <- ggplot(economics, aes(x=date, y = unemploy, colour = uempmed)) +
     geom_line()
p + scale_wrap(n=4)
}
